#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

# 导入自定义的SQL映射连接异常类
from lib.core.exception import SqlmapConnectionException
# 导入urllib库并重命名为_urllib,用于处理HTTP请求
from thirdparty.six.moves import urllib as _urllib

class HTTPRangeHandler(_urllib.request.BaseHandler):
    """
    处理HTTP Range头部的处理器类。
    Range头部允许客户端请求资源的部分内容而不是整个资源。
    
    参考文档: http://stackoverflow.com/questions/1971240/python-seek-on-remote-file
    """

    def http_error_206(self, req, fp, code, msg, hdrs):
        """
        处理206状态码(部分内容响应)
        参数说明:
        req: 原始请求对象
        fp: 类文件对象,包含响应内容
        code: HTTP状态码(206)
        msg: 响应消息
        hdrs: 响应头部信息
        """
        # 创建并返回一个包含部分内容的响应对象
        r = _urllib.response.addinfourl(fp, hdrs, req.get_full_url())
        r.code = code  # 设置响应状态码
        r.msg = msg    # 设置响应消息
        return r

    def http_error_416(self, req, fp, code, msg, hdrs):
        """
        处理416状态码(请求范围不满足)
        当请求的范围超出资源的实际范围时会触发此错误
        
        参数说明与http_error_206相同
        """
        # 构建错误信息
        errMsg = "there was a problem while connecting "
        errMsg += "target ('406 - Range Not Satisfiable')"
        # 抛出SQL映射连接异常
        raise SqlmapConnectionException(errMsg)
