#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST# 设置优先级为最低

def dependencies():
    """
    这个函数用于定义依赖关系，但在当前脚本中未实现任何功能。
    通常，这个函数用于检查当前函数所需的依赖是否满足。
    """
    pass

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，将其中的单引号字符（'）替换为其非法的双Unicode编码对应物。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    功能：
        将payload中的单引号（'）替换为%00%27，这是一种非法的Unicode编码方式，用于绕过某些安全防护措施。

    示例：
        >>> tamper("1 AND '1'='1")
        '1 AND %00%271%00%27=%00%271'
    """

    return payload.replace('\'', "%00%27") if payload else payload # 替换payload中的单引号为%00%27
