#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

import os

from lib.core.common import singleTimeWarnMessage
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST

def dependencies():
    singleTimeWarnMessage("tamper script '%s' is only meant to be run against %s" % (os.path.basename(__file__).split(".")[0], DBMS.ACCESS))
    # 显示警告信息，指出该tamper脚本仅适用于Microsoft Access数据库
def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，通过在末尾添加一个NULL字节（%00）。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    功能：
        在payload的末尾添加一个NULL字节（%00），这在对付某些弱Web应用防火墙时非常有用，特别是当后端数据库管理系统是Microsoft Access时。

    要求：
        * 仅适用于Microsoft Access数据库。

    注意：
        * 这种技术除了可以绕过Web应用防火墙外，还有其他可能的用途。

    参考链接：
        * http://projects.webappsec.org/w/page/13246949/Null-Byte-Injection

    示例：
        >>> tamper('1 AND 1=1')
        '1 AND 1=1%00'
    """

    return "%s%%00" % payload if payload else payload # 如果payload不为空，则在其末尾添加NULL字节（%00）
