#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

import string  # 导入字符串常量库，用于处理字符和十六进制数字

from lib.core.enums import PRIORITY  # 从核心库导入优先级枚举

# 设置优先级为普通
__priority__ = PRIORITY.NORMAL

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，通过Unicode转义非编码字符（不处理已经编码的字符）。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    功能：
        - 将输入的payload中的字符转换为Unicode转义序列（例如，'SELECT'转换为'\u0053\u0045\u004C\u0045\u0043\u0054'）。

    注意：
        * 这个方法对于绕过在JSON上下文中的弱过滤和/或Web应用防火墙（WAFs）很有用。

    示例：
        >>> tamper('SELECT FIELD FROM TABLE')
        '\\u0053\\u0045\\u004C\\u0045\\u0043\\u0054 \\u0020\\u0046\\u0049\\u0045\\u004C\\u0044 \\u0046\\u0052\\u004F\\u004D \\u0054\\u0041\\u0042\\u004C\\u0045'
    """

    retVal = payload  # 初始化返回值为输入的payload


    if payload:  # 如果payload不为空
        retVal = ""  # 初始化返回值字符串
        i = 0  # 初始化索引

        # 遍历payload中的每个字符
        while i < len(payload):
            # 如果当前字符是%且后面两个字符是十六进制数字（已编码的字符），则进行Unicode转义
            if payload[i] == '%' and (i < len(payload) - 2) and payload[i + 1:i + 2] in string.hexdigits and payload[i + 2:i + 3] in string.hexdigits:
                retVal += "\\u00%s" % payload[i + 1:i + 3]
                i += 3
            else:
                # 对未编码的字符进行Unicode转义，并添加到返回值
                retVal += '\\u%.4X' % ord(payload[i])
                i += 1  # 移动索引，处理下一个字符

    return retVal
