#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

from lib.core.enums import PRIORITY  # 从核心库导入优先级枚举

# 设置优先级为低
__priority__ = PRIORITY.LOW

def dependencies():
    pass

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，将所有字符转换为HTML实体（使用十进制代码点）。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    功能：
        - 遍历payload中的每个字符，并将每个字符转换为其对应的HTML实体形式（例如，' -> &#39;）。

    示例：
        >>> tamper("1' AND SLEEP(5)#")
        '&#49;&#39;&#32;&#65;&#78;&#68;&#32;&#83;&#76;&#69;&#69;&#80;&#40;&#53;&#41;&#35;'
    """

    retVal = payload  # 初始化返回值为输入的payload

    if payload:  # 如果payload不为空
        retVal = ""  # 初始化返回值字符串
        i = 0  # 初始化索引

        # 遍历payload中的每个字符
        while i < len(payload):
            # 将当前字符转换为其对应的HTML实体形式，并添加到返回值字符串
            retVal += "&#%s;" % ord(payload[i])
            i += 1

    return retVal
