#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

import os  # 导入操作系统模块，用于获取文件路径等信息
import re  # 导入正则表达式模块

from lib.core.common import singleTimeWarnMessage  # 从核心库导入单次警告消息函数
from lib.core.enums import DBMS  # 从核心库导入数据库管理系统枚举
from lib.core.enums import PRIORITY  # 从核心库导入优先级枚举

# 设置优先级为最高
__priority__ = PRIORITY.HIGHEST

def dependencies():
    singleTimeWarnMessage("tamper script '%s' is only meant to be run against %s" % (os.path.basename(__file__).split(".")[0], DBMS.ORACLE))

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，将其中的'UNION'关键字替换为'DUNION'。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    要求：
        * 仅适用于Oracle数据库。

    注意：
        * 参考文档：https://media.blackhat.com/us-13/US-13-Salgado-SQLi-Optimization-and-Obfuscation-Techniques-Slides.pdf

    示例：
        >>> tamper('1 UNION ALL SELECT')
        '1DUNION ALL SELECT'
    """
    # 如果payload不为空，使用正则表达式替换其中的UNION关键字
    return re.sub(r"(?i)(\d+)\s+(UNION )", r"\g<1>D\g<2>", payload) if payload else payload
