#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

import re

from lib.core.enums import PRIORITY  # 从核心库导入优先级枚举

# 设置优先级为最高
__priority__ = PRIORITY.HIGHEST

def dependencies():
    pass

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，将所有的等号（'='）操作符替换为'RLIKE'操作符。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    测试情况：
        * MySQL 4, 5.0 和 5.5

    注意：
        * 这个篡改方法对于绕过那些过滤等号（'='）字符的弱Web应用防火墙很有用。

    示例：
        >>> tamper('SELECT * FROM users WHERE id=1')
        'SELECT * FROM users WHERE id RLIKE 1'
    """

    retVal = payload  # 初始化返回值为输入的payload

    if payload:
        # 使用正则表达式替换payload中的等号（'='）为'RLIKE'
        # \s* 匹配任意数量的空白字符（包括0个）
        # = 匹配等号字符
        # \s* 匹配等号后的任意数量的空白字符
        retVal = re.sub(r"\s*=\s*", " RLIKE ", retVal)

    return retVal
