#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

from lib.core.enums import PRIORITY

# 设置优先级为普通
__priority__ = PRIORITY.NORMAL

def dependencies():
    pass

def tamper(payload, **kwargs):
    """
    这个函数用于篡改（tamper）输入的payload，通过斜杠转义单引号和双引号。

    参数：
        payload：要篡改的原始payload。
        **kwargs：其他可选参数（在本函数中未使用）。

    功能：
        - 将payload中的单引号（'）替换为转义形式（\\'）。
        - 将payload中的双引号（"）替换为转义形式（\\"）。

    示例：
        >>> tamper('1" AND SLEEP(5)#')
        '1\\\\" AND SLEEP(5)#'
    """
    # 替换payload中的单引号和双引号为它们的转义形式
    return payload.replace("'", "\\'").replace('"', '\\"')
