#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""
# 从sqlmap的库中导入兼容模块中的xrange函数和优先级枚举
from lib.core.compat import xrange
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW

def dependencies():
    pass

def tamper(payload, **kwargs):
    """
    Replaces space character (' ') with comments '/**/'

    Tested against:
        * Microsoft SQL Server 2005
        * MySQL 4, 5.0 and 5.5
        * Oracle 10g
        * PostgreSQL 8.3, 8.4, 9.0

    Notes:
        * Useful to bypass weak and bespoke web application firewalls

    >>> tamper('SELECT id FROM users')
    'SELECT/**/id/**/FROM/**/users'
    """

    retVal = payload

    if payload:
        retVal = "" # 初始化引号状态标记
        quote, doublequote, firstspace = False, False, False

        # 遍历payload中的每个字符
        for i in xrange(len(payload)):
            # 如果是第一个空格且之前没有遇到过空格
            if not firstspace:
                if payload[i].isspace():
                    firstspace = True
                    retVal += "/**/"
                    continue
            # 如果是单引号
            elif payload[i] == '\'':
                quote = not quote
            # 如果是双引号
            elif payload[i] == '"':
                doublequote = not doublequote
            # 如果是空格且之前没有遇到过双引号和单引号
            elif payload[i] == " " and not doublequote and not quote:
                retVal += "/**/"
                continue
            # 添加当前字符到retVal
            retVal += payload[i]

    return retVal
